@@ -1,7 +1,7 @@
 use v6;
 use fatal;
 
-unit class Getopt::Long:ver<0.1.8>;
+unit class Getopt::Long:ver<0.2.0>;
 
 role Exceptional is Exception {
 }
@@ -13,7 +13,7 @@ class Exception does Exceptional {
 	}
 }
 
-class ValueInvalid does Exceptional {
+our class ValueInvalid does Exceptional {
 	has $.format;
 	method new(Str $format) {
 		return self.bless(:$format);
@@ -182,6 +182,8 @@ my sub converter-for-format(Str:D $format) {
 	return %converter-for-format{$format} // die Exception.new("No such format $format");
 };
 
+my rule name { [\w+]+ % '-' | '?' }
+
 my grammar Argument {
 	token TOP {
 		<names> <argument>
@@ -191,7 +193,7 @@ my grammar Argument {
 	}
 
 	token names {
-		[ $<name>=[<[\w-]>+ | '?'] ]+ % '|'
+		<name>+ % '|'
 		{ make @<name>».Str.list }
 	}
 
@@ -261,6 +263,10 @@ method new-from-patterns(Getopt::Long:U: @patterns, Str :$positionals = "") {
 	return self.new(:%options, :@positionals);
 }
 
+our role Parseable {
+	method parse-argument(Parseable:U: Str $value) { ... }
+}
+
 sub get-converter(Any:U $type) {
 	state %converter-for-type{Any:U} = (
 		(Int)      => &int-converter,
@@ -286,6 +292,11 @@ sub get-converter(Any:U $type) {
 		trait_mod:<returns>(&enum-converter, $type);
 		return &enum-converter;
 	}
+	elsif $type ~~ Parseable {
+		my $callback = sub (Str $value) { return $type.parse-argument($value) };
+		trait_mod:<returns>($callback, $type);
+		return $callback;
+	}
 	else {
 		die Exception.new("No conversion known for type {$type.^name}");
 	}
@@ -372,7 +383,7 @@ method new-from-sub(Getopt::Long:U: Sub $main) {
 	return self.new(:%options, :@positionals);
 }
 
-method get-options(Getopt::Long:D: @args is copy, :%hash, :named-anywhere(:$permute) = False, :$compat-builtin = False, :$bundling = !$compat-builtin, :$compat-singles = $compat-builtin, :$compat-negation = $compat-builtin, :$compat-positional = $compat-builtin, :$write-args) {
+method get-options(Getopt::Long:D: @args is copy, :%hash, :$auto-abbreviate = False, :$compat-builtin = False, :named-anywhere(:$permute) = !$compat-builtin, :$bundling = !$compat-builtin, :$compat-singles = $compat-builtin, :$compat-negation = $compat-builtin, :$compat-positional = $compat-builtin, :$write-args) {
 	my @list;
 
 	sub wrap-exceptions(Str $description, Str $value, &action) {
@@ -406,7 +417,24 @@ method get-options(Getopt::Long:D: @args is copy, :%hash, :named-anywhere(:$perm
 		my $consumed = 0;
 
 		sub get-option($key) {
-			return %!options{$key} // die Exception.new("Unknown option $key");
+			with %!options{$key} -> $option {
+				return $option;
+			}
+			elsif $auto-abbreviate {
+				my @names = %!options.keys.grep(*.starts-with($key));
+				if @names == 1 {
+					return %!options{ @names[0] };
+				}
+				elsif @names > 1 {
+					die Exception.new("Ambiguous partial option '$key', possible interpretations: @names[]");
+				}
+				else {
+					die Exception.new("Unknown option $key");
+				}
+			}
+			else {
+				die Exception.new("Unknown option $key");
+			}
 		}
 
 		sub take-value($option, $value) {
@@ -432,7 +460,9 @@ method get-options(Getopt::Long:D: @args is copy, :%hash, :named-anywhere(:$perm
 			}
 		}
 
-		if $compat-singles && $head ~~ / ^ '-' $<name>=<alnum> '=' $<value>=[.*] / -> $/ {
+		my rule name { [\w+]+ % '-' | '?' }
+
+		if $compat-singles && $head ~~ / ^ '-' <name> '=' $<value>=[.*] / -> $/ {
 			my $option = get-option($<name>);
 			die Exception.new("$<name> doesn't take one argument") if $option.arity.max != 1;
 			take-value($option, ~$<value>);
@@ -454,16 +484,16 @@ method get-options(Getopt::Long:D: @args is copy, :%hash, :named-anywhere(:$perm
 			@list.append: |@args;
 			last;
 		}
-		elsif $head ~~ / ^ '-' ** 1..2 $<name>=[\w <[\w-]>*] $ / -> $/ {
+		elsif $head ~~ / ^ '-' ** 1..2 <name> $ / -> $/ {
 			take-args(get-option($<name>));
 		}
-		elsif $head ~~ / ^ '--' $<name>=[<[\w-]>+] '=' $<value>=[.*] / -> $/ {
+		elsif $head ~~ / ^ '--' <name> '=' $<value>=[.*] / -> $/ {
 			my $option = get-option($<name>);
 			die Exception.new("$<name> doesn't take arguments") if $option.arity.max == 0;
 			take-value($option, ~$<value>);
 			take-args($option);
 		}
-		elsif $compat-negation && $head ~~ / ^ '-' ** 1..2 '/' $<name>=[\w <[\w-]>*] ['=' $<value>=[.*]]?  $ / {
+		elsif $compat-negation && $head ~~ / ^ '-' ** 1..2 '/' <name> ['=' $<value>=[.*]]?  $ / {
 			if $<value> {
 				my $option = get-option($<name>);
 				die Exception.new("$<name> doesn't take an argument") if $option.arity.max != 1;
@@ -484,7 +514,7 @@ method get-options(Getopt::Long:D: @args is copy, :%hash, :named-anywhere(:$perm
 		}
 	}
 	@$write-args = @list if $write-args;
-	state Str @labels = flat <first second third fourth fifth sixth seventh eighth nineth tenth some some> ... *;
+	state Str @labels = <first second third fourth fifth sixth seventh eighth nineth tenth some some> ... *;
 	my &fallback-converter = $compat-positional ?? &maybe-converter !! &null-converter;
 	my @converters = |@!positionals, &fallback-converter, *;
 	@list = (@labels Z @list Z @converters).map: { wrap-exceptions(|@^args) }
@@ -660,7 +690,16 @@ It supports the following types for named and positional arguments:
 =item DateTime
 =item Date
 
-It also supports any enum type.
+It also supports any enum type, as well as any type that implements
+the Getopt::Long::Parseable role, e.g.:
+
+class FourtyTwo does Parseable {
+  has Int $.value;
+  method parse-argument(Int(Str) $value) {
+    die Getopt::Long::Invalid("Argument %s is not 42") if $value != 42;
+    return FourtyTwo.new(:$value);
+  }
+}
 
 =head2 Simple options
 
@@ -1058,9 +1097,6 @@ The following two calls behave identically:
     $ret = get-options( ... );
     $ret = get-options-from(@*ARGS, :overwrite, ... );
 
-=head2 Bundling
-
-
 =head1 Configuring Getopt::Long
 
 C<get-options> and C<get-options-from> take the following named options
@@ -1068,27 +1104,20 @@ to configure. When using Getopt::Long as a C<MAIN> wrapper, you can set
 them using the C<%*SUB-MAIN-OPTS> variable:
 
 =begin item
-permute (default:disabled)
+auto-abbreviate (default: C<False>)
 
-Whether command line arguments are allowed to be mixed with options.
-Default is disabled.
-
-If C<permute> is enabled, this means that
-
-    --foo arg1 --bar arg2 arg3
-
-is equivalent to
-
-    --foo --bar arg1 arg2 arg3
+Enabling this allows option names to be abbreviated to uniqueness (e.g.
+`--foo` can be written as `--f` if no other option starts with an `f`).
 
 =end item
 
 =begin item
-compat-builtin (default: disabled)
+compat-builtin (default: C<False>)
 
 Enable all compatibility options that make argument parsing more like
 the builtin argument parsing. Currently that means disabling C<bundling>
-and enabling C<compat-singles>, C<compat-negation> and C<compat-positional>.
+and C<permute>, and enabling C<compat-singles>, C<compat-negation> and
+C<compat-positional>.
 
 =end item
 
@@ -1099,7 +1128,7 @@ Enabling this option will allow single-character options to be
 bundled. To distinguish bundles from long option names, long options
 I<must> be introduced with C<--> and bundles with C<->.
 
-Note that, if you have options C<a>, C<l> and C<all>, , possible
+Note that, if you have options C<a>, C<l> and C<all>, possible
 arguments and option settings are:
 
     using argument   sets option(s)
@@ -1111,6 +1140,22 @@ arguments and option settings are:
 
 =end item
 
+=begin item
+permute (default: C<!$compat-builtin>)
+
+Whether command line arguments are allowed to be mixed with options.
+Default is enabled unless C<$compat-builtin> is set.
+
+If C<permute> is enabled, this means that
+
+    --foo arg1 --bar arg2 arg3
+
+is equivalent to
+
+    --foo --bar arg1 arg2 arg3
+
+=end item
+
 =begin item
 compat-singles (default: C<$compat-builtin>)
 
@@ -1140,7 +1185,8 @@ possible.
 
 =head1 Return values and Errors
 
-C<get-options> returns a capture to indicate success.
+C<get-options> returns a capture to indicate success, or throws an
+C<Getopt::Long::Exceptional> otherwise.
 
 =head1 Troubleshooting
 
